/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.datatypes;

import java.text.*;

/**
  * An immutable data type representing Rate.
  */
abstract public class Decimal extends Number
  {
  private DecimalFormat df = null;

  private long _internal;

  public Decimal()
     {
     this(0);
     }

  public Decimal(Decimal value)
     {
     this(value.getDouble());
     }
     
  public Decimal(double value)
     {
     _internal = _toInternal(value);
     }

  public Decimal(int value)
     {
     _internal = value * getFactor();
     }

  public Decimal(String value)
     {
     this(Double.parseDouble(value));
     }

  abstract protected int getFactor();
  abstract protected int getDecPlaces();
  abstract protected String getFormatMask();



  public double getDouble()
     {return _internal > 0 ? ((double) _internal) / getFactor() : 0;}

  public int getInt()
     {return _internal > 0 ? (int)  _internal / getFactor() : 0;}
     
  public String toString()
     {
     String dollars = "" + (int) (_internal / getFactor());
     String cents =  "" + (int)(_internal % getFactor());
     while(cents.length() < getDecPlaces())
        cents = cents + "0";

     return "" + dollars + "." + cents;
     }

  public String getString()
     {return toString();}
     
  public String getFormattedString()
     {return getDecimalFormatter().format(getDouble());}

  public Decimal add(double amount)
     {
     _internal += _toInternal(amount);
     return this;
     }

  public Decimal add(Decimal decimal)
     {
     _internal += _toInternal(decimal.getDouble());
     return this;
     }

  public Decimal subtract(double amount)
     {
     _internal -= _toInternal(amount);
     return this;
     }

  public Decimal subtract(Decimal decimal)
     {
     _internal -= _toInternal(decimal.getDouble());
     return this;
     }

  public Decimal times(int value)
     {
     _internal *= value;
     return this;
     }

  public Decimal dividedBy(int value)
     {
     if(value == 0)
        throw new IllegalArgumentException("Attempt to divide by zero");
     _internal = (long) (((double) _internal / value) + 0.5);
     return this;
     }

   public boolean greaterThan(Decimal decimal)
     {
     return (this.getDouble() > decimal.getDouble()) ;
     }

   public boolean lessThan(Decimal decimal)
     {
     return (this.getDouble() < decimal.getDouble()) ;
     }

   public boolean notEqual(Decimal decimal)
     {
     return (this.getDouble() != decimal.getDouble()) ;
     }

   public boolean equals(Decimal decimal)
     {
     return (this.getDouble() == decimal.getDouble()) ;
     }

   private long _toInternal(double amt)
      {
      return (long) Math.floor((amt * getFactor()) + 0.5d);
      }

  // abstract method implementations that are defined
  // in Number class
   public double doubleValue()
     {return getDouble();}
   public float floatValue()
     {return (float) getDouble();}
   public int intValue()
     {return getInt();}
   public long longValue()
     {return _internal > 0 ? _internal / getFactor() : 0;}

   private DecimalFormat getDecimalFormatter()
      {
      if(df == null)
         df = new DecimalFormat(getFormatMask());
      return df;
      }

   final protected static String templateBuilder(String s, int places)
       {
       while(places-- > 0)
          s = s + "0";
       return s;
       }
   }





