/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */


package osmq.mcns;

import java.net.*;
import java.io.*;

class MCSocket extends MulticastSocket
     {
     private int _savgrp;
     private InetAddress _savaddress = null;
     
     /**
       * Default TTL is set to 3.
       */
     public MCSocket(int group, int ttl)  throws IOException
        {
        super(group);
        _savgrp = group;
        this.setTimeToLive(ttl > 0 ? ttl : NetworkScope.getScope().getTtl());
        }
        
     /**
       * If already belonging to a group, the method causes
       * the socket to leave the current group before joining
       * another.
       */
     public void joinGroup(InetAddress ga) throws IOException
        {
        if(_savaddress != null)
           this.leaveGroup(_savaddress);
        super.joinGroup(ga);
        _savaddress = ga;
        }

     public void  leaveGroup(InetAddress ga) throws IOException
        {
        try{super.leaveGroup(ga);}
        catch(IOException e){throw e;}
        finally{_savaddress = null;}
        }

     public void close()
        {
        try{
           if(_savaddress != null)
              this.leaveGroup(_savaddress);
           }
        catch(IOException x){}
        _savaddress = null;
	      super.close();
        }
        
     protected void finalize() throws Throwable
        {this.close();}
     }
