/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.mcns;


import java.net.*;
import java.io.*;

/**
  * Utility bean that listens for a datagram on a predefined multicast
  * group/port combination. <P>
  * The class has a public member function (getNextPacket())
  * that can be called iteratively to retrieve datagrams.
  * An instance of this class is used by AbstractServer
  */
public class MulticastListener
  {
  private InetAddress groupaddress;
  private int channel = MCNS.DISCOVERY_CHANNEL;  // can be changed by user

  private String          _dottedgroupaddress;
  private int             _ttl;

  private MulticastSocket _socket;

  private DatagramPacket  _packet;
  private boolean         _isopen = false;

  private boolean         _timedout = false;
  private boolean         _emptypacket = false;

  public  byte            _buffer[] = new byte[MCNS.READBUF_SIZE];
  private int             timeout = 0;
  private int             remoteport;
  private String          remoteData;
  private String          remoteaddressString;     // address found in the transmitted packet
  private InetAddress     remoteaddress;

  MulticastListener()
     {
     _dottedgroupaddress = NetworkScope.getScope().getGroupAddress();
     _ttl = NetworkScope.getScope().getTtl();
     }

  public void setChannel(int channel)
     {this.channel = channel;}
  public int getChannel()
     {return channel;}

  public void setDottedGroupAddress(String dottedaddress)
     {_dottedgroupaddress = dottedaddress;}

  public String getDottedGroupAddress()
     {return _dottedgroupaddress;}

  public void setTimeoutSecs(int v)
     {timeout = v;}
  public int getTimeoutSecs()
     {return timeout;}

  public boolean isTimedOut()
     {return _timedout;}
     
  public boolean isEmptyPacket()
     {return _emptypacket;}

  public byte[] getBuffer()
     {return _buffer;}

  public void open() throws IOException
      {
      try
         {
         // the socket listens to a specific channel (port) with a time-to-live
         // value (number of router hops)
         _socket = new MCSocket(channel, _ttl);
         
         // Timeout on a listen if one has been set
         if(timeout > 0)
             _socket.setSoTimeout(timeout * MCNS.SECONDS);

         groupaddress = InetAddress.getByName(_dottedgroupaddress);
         
         // join the multicast group (ip address)
         _socket.joinGroup(groupaddress);

         _isopen = true;
         }
         
      catch(IOException e)
        {
        try{_socket.leaveGroup(groupaddress);}
        catch(IOException x){}
	      _socket.close();
        _socket = null;
        _isopen = false;
        throw e;
        }
      }

  public String getRemoteAddressString()
     {return remoteaddressString;}
      
  public int getRemotePort()
     {return remoteport;}

  public String getRemoteData()
     {return remoteData;}

  public void setTimeToLive(int ttl)
     {_ttl = ttl;}

  public int getTimeToLive()
     {return _ttl;}

  public InetAddress getRemoteAddress()
     {return remoteaddress;}

  final public void getNextPacket() throws IOException
     {
     if(!_isopen)
        this.open();
     _timedout = _emptypacket = false;
     _packet = new DatagramPacket(_buffer, _buffer.length);

     try{_socket.receive(_packet);}
     catch(InterruptedIOException e)
         {
         _timedout = true;
         return;
         }
     catch(IOException e)
         {
         close();
         throw e;
         }

      if(_packet.getLength() == 0)
         {
         _emptypacket = true;
         return;
         }

      remoteData = new String(_packet.getData(), 0, _packet.getLength());

      remoteaddress = _packet.getAddress();

      remoteport          = _packet.getPort();
      remoteaddressString =  remoteaddress.getHostAddress();
     }

  public void close()
      {
      if(!_isopen)
         return;
      if(_socket != null)
        {
        try{_socket.leaveGroup(groupaddress);}
        catch(IOException e){}
	      _socket.close();
        _socket = null;
        _isopen = false;
        }
      }

  // SAFETY TO FORCE CLOSE DURING GARBAGE COLLECTION    
  protected void finalize() throws Throwable
    {close();}

  public static MulticastListener createListener()
     {
     return new MulticastListener();
     }
  }

  
