/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */
 
package osmq.mcns;

public class NetworkScope
  {
  final public static String LOCAL_DOTTED_IP_GROUP  = "239.10.10.10";
  final public static String EXTENDED_DOTTED_IP_GROUP = "238.50.34.15";

  final public static int LOCAL_TTL = 1;
  final public static int EXTENDED_TTL = 3;

  final public static int LAN = 0;
  final public static int WAN = 1;

  private int    _scope = LAN;
  private int    _ttl = -1;
  private String _ipgroup = null;

  private static NetworkScope singleton = new NetworkScope(LAN);

  private NetworkScope(int scope)
     {_scope = scope;}

  public boolean isWan(){return _scope == WAN;}
  public boolean isLan(){return _scope == LAN;}

  public String getGroupAddress()
     {
     if(_ipgroup != null)
        return _ipgroup;
     return (isLan()) ?
         LOCAL_DOTTED_IP_GROUP :
         EXTENDED_DOTTED_IP_GROUP;
     }

  public void setGroupAddress(String dottedaddress)
     {_ipgroup = dottedaddress;}

  public void setTtl(int ttl)
     {_ttl = ttl;}
     
  public int getTtl()
     {
     if(_ttl > -1)
        return _ttl;
     return (isLan()) ? LOCAL_TTL : EXTENDED_TTL;
     }

  public static NetworkScope setLanScope()
     {
     singleton = new NetworkScope(LAN);
     return singleton;
     }

  public static NetworkScope setWanScope()
     {
     singleton = new NetworkScope(WAN);
     return singleton;
     }

  public static NetworkScope getScope()
     {return singleton;}
  }