/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */


package osmq.messages;
import osmq.logs.Logs;
import osmq.util.TokenFinder;


public class MessageByteSplitter
  {
  private static MessageByteSplitter _mbs;
  private int occurance = 0; // relative delimiter

  /**
    * Default constructor uses the default number of header elements
    * contained in the derived class (based on getDefaultSize()
    */
  public MessageByteSplitter()
    {
    // default is message with standard size header 
    this(MessageHeader.TOTAL_ATTRIBUTES + 1);
    }

  /**
    * Constructor that overrides the default number of header elements.
    */
  public MessageByteSplitter(int occurance)
     {
     setOccurance(occurance);
     }

  private void setOccurance(int occurance)
     {
     if(occurance < 1)
        throw new IllegalArgumentException("occurance must be > 0");
     this.occurance = occurance;
     }

   final public String[] split(byte[] source, String[] target) //throws ERMessageException
     {
     int posn ;

     if(source.length < 1)
        throw new IllegalArgumentException("Zero length string");
        
     if(target.length < 2)
        throw new IllegalArgumentException("target has less than two elements");

     // Find the position of the relative delimiter that begins the second string.
     // Normally, this is the delimiter that begins the body section.
     posn = findTokenOccurance(source, source[0], occurance);
     
     if(posn == -1)
        {
        throw new IllegalArgumentException("Invalid string in StringSplitter");
        }
     target[0] = new String(source, 0, posn);
     target[1] = new String(source, posn, source.length - posn);

     return target;
     }

  /**
      * Locates an occurance of a character within a string and
      * returns its offset.  The occurances are numbered 1-n.
      * If the occurance is not found, returns -1.
      * If found, returns the zero-relative offset position of the character.
      */
    public final int findTokenOccurance(byte[] s, byte token, int _occurance)
     {
     int _lastfound  = -1;
     int _foundcount = 0;
     int _offset     = 0;
     int _lgh = s.length;

     while(_offset < _lgh)
         {
         if(s[_offset] == token)
            {
            ++_foundcount;  // found another occurance
            if(_foundcount == _occurance)  // if last one
               {
               _lastfound = _offset;
               return _offset;           // return its zero-relative offset postion
               }
            }
         _offset++;
         }
     return -1;
     }

  public static MessageByteSplitter getByteSplitter()
      {
      if(_mbs == null)
         {
         synchronized(MessageByteSplitter.class)
            {
            if(_mbs == null)
               _mbs = new MessageByteSplitter();
            }
         }
      return _mbs;
      }

  public static void main(String[] args)
     {
     String[] aftersplit = new String[2];
     String mystring = "|one||three|four||six|seven||nine|ten|eleven|twelve";
     MessageByteSplitter s = new MessageByteSplitter(MessageHeader.TOTAL_ATTRIBUTES + 1);
     aftersplit = s.split(mystring.getBytes(), aftersplit);
     int i = new TokenFinder().findTokenOccurance(mystring, mystring.charAt(0), 5);
     System.out.println(mystring.substring(i));
     System.out.println(aftersplit[0]);
     System.out.println(aftersplit[1]);
     }
  }
