/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.messages;

import java.io.*;
import java.util.*;

/**
  * Supports differentiating MessageListener instance by topic.
  */
public class TopicsListeners  implements MessageListener
  {
   private Map             _map = new HashMap();
   private MessageListener _defaultlistener = null;

   public TopicsListeners(){}
   
   public void setDefaultListener(MessageListener listener)
      {_defaultlistener = listener;}

   public MessageListener getDefaultListener()
      {return _defaultlistener;}

   public void add(String topic, MessageListener listener)
      {_map.put(topic, listener);}

   public boolean hasTopic(String topic)
      {return _map.containsKey(topic);}

   public void delete(String topic)
      {_map.remove(topic);}

   public int getCount()
      {return _map.size();}
      
   public void onMessage(osmq.messages.Message cm)
      {
      if(cm.hasAssignedRecipient())
         {
         if(_defaultlistener != null)
            _defaultlistener.onMessage(cm);
         }
      else
         {
         // Look up the topic in the map, and if found, pass the
         // message to the corresponding MessageListener
         MessageListener listener = (MessageListener) _map.get(cm.getTopic());
         if(listener != null)
            listener.onMessage(cm);
         }
      }
   }