/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */


package osmq.mque;

import java.io.*;
import java.io.Serializable;
import java.util.*;
import osmq.util.*;

class MemoryFifo extends ReadableFifo implements Serializable
  {
  private boolean clonebytes = false;
  private long  pulled = 0;
  protected ArrayList list = new ArrayList();
  byte[] rb;

  public MemoryFifo() {}


  public void writeBytes(byte[] value) throws IOException
     {
     ensureOpen();
     if(value == null || value.length == 0)
         throw new IOException("Null byte array");
     if(clonebytes)
         {
         byte[] target = new byte[value.length];
         System.arraycopy(value, 0, target, 0, value.length);
         list.add(target);
         }
      list.add(value);
     incrementCount();
     }

   /**
    * Returns the next object (FIFO order) from the Queue, and removes that
	  * object from the Queue. If the Queue is empty, the function blocks until
    * it is notified that an object is available.
    * @return the next object from the Queue (FIFO order), or null if none is waiting.
    * @exception QueueException thrown if Queue is in a closed state AND the Queue is empty.
    * @exception QueueException thrown if Queue is in a <i>halt-reads</i> state.
    * @see #isClosed()
    * @see #isHaltReads()
    */
  public byte[] readBytes() throws IOException
     {
     ensureOpen();
     if(size() < 1)
        throw new QueueException("Fifo is empty");
     rb = (byte[]) list.remove(0);
     if(rb == null || rb.length < 1)
        throw new QueueException("Null byte array in Fifo");
     decrementCount();
     ++pulled;
     return rb;
     }

  public long getPulledCount()
     {
     return pulled;
     }

  /**
    * Sets the Queue's state to <i>closed</i> so that no objects can be
    * written to the fifo.
    * @see #isClosed()
    */
  protected void closeImpl()
     {
     list.clear();
     list = null;
     }

  /**
    * Removes all the elements from the Queue.
    */
  public void clear() throws IOException
     {
     ensureOpen();
     list.clear();
     }

  private void ensureOpen() throws IOException
     {
     if(isClosed())
        throw new QueueException("Fifo is closed",
                                 QueueException.QE_CLOSED);
     }

  public void setCloneBytes(boolean b)
     {
     clonebytes = b;
     }

  public boolean isCloneBytes()
     {
     return clonebytes;
     }
  }
