/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */


package osmq.mque;

import java.util.*;
import java.io.*;

/**
  * A Queue implementation that utilizes a persistent data file
  */
abstract class ReadableFifo
  {
  private boolean _closed = false;
  private long    _count = 0;

  abstract public byte[] readBytes() throws IOException;
  abstract protected void closeImpl();
    /**
    * Returns <i>true</i> if the Queue is in an <i>open</i> state so that
    * reads and writes can be performed.  Otherwise returns <i>false</i>.
    * @see #close()
    */
  final public boolean isOpen(){return !_closed;}

  /**
    * Returns <i>true</i> if Queue is in a <i>closed</i> state.
    * If a Queue is in a <i>closed</i> state,
    * <p> 1. no writes can be performed (they will throw a QueueException), and
    * <p> 2. reads can be performed until the Queue is emptied, at which point
    *        a read will throw a QueueException.
    */
  final public boolean isClosed(){return _closed;}


  final public void close()
     {
     if(!_closed)
        {
        closeImpl();
        _closed = true;
        }
     }
     
  /**
    * Returns the number of objects currently in the Queue.
    */
  final public long size()
     {
     return _count;
     }

  public boolean isEmpty()
     {
     return _count == 0;
     }

  protected void incrementCount()
     {
     _count++;
     }

  protected void decrementCount()
     {
     _count--;
     }

  }

