/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */
 

package osmq.pulsar;
  
import osmq.util.*;
import osmq.util.StateEventBean;
import osmq.util.StateEventListener;
import osmq.mcns.MulticastListener;
import osmq.mcns.MCNS;
import osmq.util.Threaded;


import java.net.*;
import java.io.*;

/** An aggregate class used by a service's client to identify if and when a service fails.
  * The service being monitored must employ a HeartbeatTransmitter -- a ServicePulsar
  * transmitting at the standard "heartbeat" frequency.
  * The client class registers a function to be called if / when the service fails.
  */

public class HeartbeatListener extends Threaded
  {
  private StateEventBean eb = new StateEventBean();
  private boolean shouldExit = false;
  private boolean timeoutfatal = true;
  private MulticastListener listener =  MulticastListener.createListener();
  
  // track accumulated time for ignoring false pulses
  private StopWatch sw = new StopWatch();
  private int listenport = MCNS.PULSAR_CHANNEL;
  private boolean servicefailure = false;

  public static int buffer_size = 1024;
  private String service = "Heartbeat"; 
  public static byte buffer[] = new byte[buffer_size];

  public HeartbeatListener()
    {listener.setTimeoutSecs(Threshold.TIMEOUT_THRESHOLD);}

  public void setTimeoutSeconds(int s)
    { listener.setTimeoutSecs(s);}
  public int getTimeoutSeconds()
    {return listener.getTimeoutSecs();}

  public void setPulsarPort(int p)
    {listenport = p;}
  public int getPulsarPort()
    {return listenport;}

  public  void addListener(StateEventListener l)
    {eb.addStateEventListener(l);}

  public  void removeListener(StateEventListener l)
    {eb.removeStateEventListener(l);}

 /**
   * Causes the ServicePulsarMonitor to begin listening for the pulse of a service.
   * If it detects a pulse lapse greater than a timeout value
   * (timeout value defaults to Multicasters.PulsarAlarmThreshold in seconds), it
   * will notify all classes that have registered as FatalEventListener instances.
   */
  final protected void initializer()  throws Exception
    {
    if(service == null)
       throw new Exception("Service not set in HeartbeatListener.class");
    servicefailure = false;
    listener.setChannel(listenport);
    sw.reset();
    System.out.println("Starting HeartbeatMonitor for port " + listenport);
    }

  final protected void iterativeProcess()  throws Exception
     {
     listener.getNextPacket();
          
     if(listener.isTimedOut())
        {
        // NO PULSE OF ANY SORT ON THE PORT
        servicefailure = true;
        System.out.println("Service timeout for " + service);
        throw new Exception("Service timeout for " + service);
        }
             
     // VERIFY PULSE FROM THE CORRECT SERVICE
      if(listener.getRemoteData().equals(service))
        {
        // HEARD CORRECT PULSE W/I TIME LIMIT, SO LOOP BACK TO TEST AGAIN
        sw.reset();
        return;
        }
             
     // PULSE DETECTED BUT NOT FROM CORRECT SERVICE,
     // SO IS CUMULATIVE WAIT GREATER THAN TIMEOUT THRESHOLD?
     if(sw.getDurationSeconds() > listener.getTimeoutSecs())
        {
        servicefailure = true;
        throw new Exception("Service timeout for " + service + " on port " + listenport);
        }
     }

  
  final protected void cleanup()
     {
     listener.close();
     if(servicefailure)
        eb.fireTimeEvent();
     }

} 
