/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */



package osmq.pulsar;
  
import java.io.*;
import java.net.*;
import java.util.*;
import osmq.mcns.*;
import osmq.net.DiscoveryClient;

class PulsingSocketServer 
   {
   private ServerSocket svs = null;
   private int port;
   private String servicename;
   private boolean master = true;
   private ServiceAlertTransmitter sapulsar;
   private String service = "";
   private HeartbeatTransmitter mp = null;

   protected PulsingSocketServer(int port, String servicename) throws IOException
      {
      svs = new ServerSocket(port);
      this.port = port;
      this.servicename = servicename;
      initAlert();
      }

   public ServerSocket getSocket()
      {return svs;}

   private void initHeartbeat()
      {
      mp = new HeartbeatTransmitter(port);
      try{mp.start();}
      catch(Exception e){}
      }
      
   private void initAlert() throws IOException
     {
     // make certain there is a service name
     if(servicename == null ||
        servicename.length() < 1)
           return;
      if(DiscoveryClient.getSingleton().isActiveService(servicename))
        {
        System.out.println("Server for " +  servicename + " already running");
        return;
        }
        
     // create a pulsar that will be monitored by MCNS
     sapulsar = new ServiceAlertTransmitter(servicename, this.port);
     try{sapulsar.start();}
     catch(Exception e){throw new IOException(e.getMessage());}

     // create a heartbeat that will be monitored by client
     initHeartbeat();
     }


   public  void close() throws IOException
      {
      try{
         if(sapulsar != null)
            sapulsar.stop();
         sapulsar = null;
         
         if(mp != null)
           mp.stop();
         mp = null;
         }
      catch(Exception e){}
      svs.close();
      }
}


