/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */

package osmq.records;
import osmq.util.ImmutableException;
import osmq.datatypes.DataType;

public class TaggedDataRecord extends DataRecordBean
    {
    private String[] tags;
    private String [] datatypes;
    
    public TaggedDataRecord()
       {
       tags = null;
       datatypes = null;
       }
       
    public TaggedDataRecord(int size)
       {
       super(size);
       allocateArrays();
       }

    public TaggedDataRecord(String value)
       {
       super(value);
       allocateArrays();
       }

    public void setText(String text)
       {
       super.setText(text);
       allocateArrays();
       }
       
    private void allocateArrays()
       {
       tags = new String[this.getElementCount()];
       datatypes = new String[this.getElementCount()];
       }

    public void setDataType(int posn, String type)
       {
       this.throwIfImmutable();
       datatypes[posn - getOffsetIndex()] = type;
       }

    public void setDataType(int posn, int type)
       {
       this.throwIfImmutable();
       datatypes[posn - getOffsetIndex()] = getAsDataType(type);
       }

    public String getDataType(int posn)
       {
       return datatypes[posn - getOffsetIndex()];
       }

    public void setTag(int posn, String tag)
       {
       this.throwIfImmutable();
       tags[posn - getOffsetIndex()] = tag;
       }

    public String getTag(int posn)
       {
       return tags[posn - getOffsetIndex()];
       }

    public String getAllDTD()
       {
       StringBuffer buffer = new StringBuffer();
       for(int i = 0 ; i <  getElementCount(); ++i)
          {
          buffer.append(getXmlDTD(i + getOffsetIndex()));
          buffer.append("\n");
          }
       return new String(buffer);
       }

    public String getAllXmlValues()
       {
        StringBuffer buffer = new StringBuffer();
       for(int i = 0 ; i <  getElementCount(); ++i)
          {
          buffer.append(getXmlValue(i + getOffsetIndex()));
          buffer.append("\n");
          }
       return new String(buffer);
       }
       
    public String getXmlDTD(int posn)
       {
         return new String("<simpleType name='" +
                               getTag(posn) +
                              "' base='" + getDataType(posn)  +
                              "'>" +
                              "\n" +
                              "  </simpleType>"
                              );
       }

    public String getXmlValue(int posn)
       {

       return new String("<" + getTag(posn) + ">" +
                         getString(posn) +
                         "</" + getTag(posn) + ">");
       }


    public void resetSize(int newsize) throws ImmutableException
	    {
      super.resetSize(newsize);
      tags = new String[newsize];
      datatypes = new String[newsize];
      }

    final static protected String getAsDataType(int type)
      {
      switch(type)
        {
        case DataType.UNKNOWN:
        case DataType.STRING:
        case DataType.CHAR:
           return "string";

        case DataType.LONG:
           return "long";

        case DataType.SIZE:
           return "long";

        case DataType.INTEGER:
           return "integer";

        case DataType.DOUBLE:
           return "double";

        case DataType.DECIMAL:
           return "decimal";

        case DataType.CURRENCY:
           return "double";
       
        default:
           return "";
        }
     }

    public static void main(String[] args)
        {
        TaggedDataRecord tds = new TaggedDataRecord("|Bob|27");
        tds.setTag(1, "Name");
        tds.setDataType(1, DataType.STRING);
        tds.setTag(2, "Age");
        tds.setDataType(2, DataType.INTEGER);
        System.out.println(tds.getAllDTD());
        System.out.println(tds.getAllXmlValues());
        System.out.println("");
        //System.out.println(tds.getXmlDTD(1));
        //System.out.println(tds.getXmlDTD(2));

        //System.out.println(tds.getXmlValue(1));
        //System.out.println(tds.getXmlValue(2));
        }
} 