/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */


package osmq.sample;

import java.util.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.io.IOException;
import java.text.DecimalFormat;

import osmq.mcns.*;
import osmq.util.*;
import osmq.records.DataRecord;
import osmq.mque.*;

import osmq.agents.NetReader;

   public class TestReader implements ExceptionListener, TickEventListener
   {
    TickEventBean teb;
    String withCommas = "###,###,###,###,###";
    DecimalFormat myformat;
    String s;
    NetReader _netreader;
    int SHOW_COUNT = 100000;
    long totalLength;
    EventRateMgr _erm;

   public TestReader(){}
   public void onException(Exception e)
      {
      System.out.println("Exception in TestSub " + e.getMessage());
      }

   public void onTickEvent(TickEvent e)
      {
       System.out.println(s);
       System.out.println("avg rate = " + _erm.getRatePerSec() + "/sec");
       System.out.println("Total count = " + myformat.format(_erm.getCount()));
       System.out.println("Bytes read = " + myformat.format(totalLength));
       System.out.println("Total pub connections = " + _netreader.getConnectionCount());
      }

   public void run()
    {
    myformat = new DecimalFormat(withCommas);
    _erm = new EventRateMgr();
    totalLength = 0;


    try{
       // Instantiate the network message reader
       _netreader = new NetReader();

       // Notify my onException(Exception) method if there is
       // a network error
       _netreader.setExceptionListener(this);


       // Identify my unique network service name for writers to dynamically locate
       _netreader.setServiceName("SAFESTORE");

       // Define my listening socket port
       _netreader.setPort(3843);

       // If the queue reaches 1000 records, overflow remainder to disk
       _netreader.setOverflowSize(1000);

       // Open the connection for message reception and begin receiving messages
       // in the queue
       _netreader.open();

        System.out.println("Message reader opened");

       // Get the first message as a string
       s = _netreader.getNextString();

       // Initialize the event rate object
       _erm.reset();

       // TickEventBean used to periodially call the onTickEvent() function
       // that will display statistics
       teb = new TickEventBean();
       teb.setAlertInterval(10);
       teb.addTickEventListener(this);
       teb.begin();

       // infinite loop to process inbound messages
       for(;;)
          {
          // update key statistics
          totalLength += s.length();
          // increment the rate counter for later display
          _erm.incrementCount();
          
          // get the next message as a string
          s = _netreader.getNextString();
          }
       }

    catch(Exception e)
       {
       System.out.println("Peer Subscriber Exception: " +
                          e.getMessage());
       try{System.in.read(new byte[10]);}
       catch(Exception ignore){}
       }
    }

    public static void main(String[] args)
          {
          TestReader ts = new TestReader();
          ts.run();
          try{System.in.read(new byte[10]);}
          catch(Exception ignore){}
          }
    }






