/* ====================================================================
 *
 *
 * Copyright (c) 1999-2002 CPortals Systems, Inc.  All rights reserved.
 *
 * This file is part of OSMQ.
 *
 *   OSMQ is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   OSMQ is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with OSMQ; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * OSMQ is a trademark of CPortals Systems, Inc.
 * ====================================================================
 */
 
package osmq.util;


import java.io.Serializable;

public class ImmutableImpl implements Immutable, Cloneable, Serializable
   {
   private boolean immutable = false;

   public ImmutableImpl()
      {}

   /**
     * Returns true if this is an immutable object.  The attributes and values
     * of an immutable object cannot be changed, and attempts to modify them
     * results in an ImmutableException being thrown.
     */
   public boolean isImmutable()
      {return immutable;}

   public void setImmutable()
       {immutable = true;}

   protected void throwIfImmutable() throws ImmutableException
      {
      if(isImmutable())
         throw new ImmutableException();
      }

   protected Object passIfNotImmutable(Object passed) throws ImmutableException
        {
        throwIfImmutable();
        return passed;
        }

   protected Object clone()
	 {
	 try{
        ImmutableImpl ii = (ImmutableImpl) super.clone();
        ii.immutable = this.immutable;
        return ii;
        }
	 catch(CloneNotSupportedException e){return this;}
	 }
   }